package Exposure;
our $VERSION = '0.20';

require 5.001;
use strict;
use warnings;
use Image::ExifTool;

use base 'Exporter';
our @EXPORT = qw( get_shutter get_iso get_aperture
  get_whitebalance do_exposure get_settings exposure_from_settings
  exposure_from_exif);

# FIXME: Is it really a good idea to use die(), or is it better
# to just return nothing?

sub get_shutter() {
	my $exif = shift;
	my $shutter;

	$shutter = $$exif{ShutterSpeedValue} || $$exif{ExposureTime} || $$exif{ShutterSpeed};
	if (! $shutter) { return; }    # Can't determine shutter speed

	if ( $shutter =~ m/^(\d+)([\.\/])?(\d+)?$/ ) {
		$shutter = $1;
		if ( defined $3 ) {
			my $tmp = $3;
			if ( $2 =~ m/\// ) {
				# e.g. 6/10
				$shutter /= $tmp;
				$shutter = sprintf "%g", $shutter;
			}
			else {
				# e.g 0.6
				$shutter .= ".$tmp";
			}
		}
		return $shutter;
	}

	return;
}

sub get_aperture() {
	my $exif = shift;
	my $aperture;

	$aperture = $$exif{FNumber} || $$exif{ApertureValue};
	if (! $aperture) { return; }    # Can't determine aperture

	if ( $aperture =~ m/^(\d+)\.(\d+)?$/ ) {
		$aperture = $1;
		if ( defined $2 ) {
			$aperture .= ".$2";
		}
		return $aperture;
	}

	return;
}

sub get_iso() {
	my $exif = shift;
	my $iso;

	$iso = $$exif{ISO} || $$exif{CCDISOSensitivity};
	if (! $iso) { return; }

	if ( $iso =~ m/^(\d+)$/ ) {
		return $1;
	}

	return;
}

sub get_whitebalance() {
	my $exif = shift;
	my $wbal;

	$wbal = $$exif{WhiteBalance};
	if (! $wbal) { return; }

	if ( $wbal =~ m/^([\w ]*)$/ ) {
		return $1;
	}

	return;
}

sub do_exposure() {
	# FIXME: Depricated. Use exposure_from_* instead.
	my $shutter  = shift;
	my $aperture = shift;
	my $iso      = shift;

	return log( $aperture * $aperture / $shutter * $iso / 100 ) / log(2);
}

sub get_settings() {
	my $exif = shift;

	my $shutter = &get_shutter( $exif ) or return;
	my $aperture = &get_aperture( $exif ) or return;
	my $iso = &get_iso( $exif ) or return;
	my $white = &get_whitebalance( $exif ) or return;

	my %settings = (
		shutter => $shutter,
		aperture => $aperture,
		iso => $iso,
		white => $white,
	);

	return %settings;
}

sub exposure_from_settings() {
	my %s = @_;

	# Exposure = f(Shutter_speed, Aperture, ISO speed):
	# EV = log2(aperture2 x (1/shutter speed) x (ISO sensitivity/100))
	# log base2 (x) = log (x) / log (2)

	return unless $s{aperture};
	return unless $s{shutter};
	return unless $s{iso};
	return unless ( $s{shutter} > 0 );

    my $exposure = log( $s{aperture} * $s{aperture} / $s{shutter} *
			$s{iso} / 100 ) / log(2);
	$exposure = sprintf "%.5f", $exposure;    # Round to 5 places

	return $exposure;
}

sub exposure_from_exif() {
	my $exif = shift;

	my %settings = &get_settings( $exif ) or return;
	my $exposure = &exposure_from_settings( %settings ) or return;

	return $exposure;
}

1;

################################################################################
#
# ChangeLog
# ---------
#
# 0.21,   26 Feb 2012: Added version string at 0.20; Removed generic EXIF stuff.
# 20110402 tidied up, improved robustness
# 20110205 merged exposure.pl from /usr/local/sbin and /var/www/cgi-bin
# 20070221 converted to Perl module
# 20060428 Initial version, based on a copy of webhdr_main.cgi
#
