package ImageLDR;

use strict;
use warnings;
use Image::Magick;

our $VERSION = "0.02";

use base 'Exporter';
our @EXPORT = qw(verify_sequence sort_brightness get_average get_greyest_idx);

sub verify_sequence() {
	my ($width_prev, $height_prev);

	my $img = Image::Magick->new;
	foreach (@_) {
		my ($width, $height, $size, $format) = $img->Ping($_);

		return ("Images are not JPEG images.")
				unless($format =~ /^JPEG$/);

		# Check that all images have the same dims
		if(defined $width_prev) {
			return ("Images are not of the same width.")
					unless($width_prev == $width);
		}

		$width_prev = $width;
		if(defined $height_prev) {
			return ("Images are not of the same height.")
					unless($height_prev == $height);
		}
		$height_prev = $height;
	}
	# All return values are errors. Well done if you are here.
	return;
} # verify_sequence

sub get_average() {
	my $img = shift;

    my $image = Image::Magick->new;
    my $err = $image->Read("$img");
	$image->Set(colorspace=>'Gray');
	my $info = $image->Get('format', '%[fx:image.mean]');

	return $info;
} # get_average

sub sort_brightness() {
	my %averages = ();
	my @sorted = ();
	my @brights = ();

	foreach (@_) {
		$averages{ "$_" } = &get_average( $_ );
	}

	my $key;
	foreach $key (sort { $averages{$b} <=> $averages{$a} } keys %averages)
	{
		push( @sorted, $key );
		push( @brights, $averages{$key} );
	}

	return \@sorted, \@brights;   # 2 arrays: file names, brightness 0..1
} # sort_brightness

sub get_greyest_idx() {
	my $min = 1;
	my $idx = 0;
	my $i = 0;

	foreach (@_) {
		my $diff = abs($_ - 0.5);
		if ($diff < $min) {
			$min = $diff;
			$idx = $i;
		}
		$i++;
	}

	return $idx;
} # get_greyest_idx

sub makehistogram(){
    # TODO: tidy this up; use File::Temp
	#my $img   = "${pickup_dir}${timestring}_hdr.jpg";
#    my $pnm   = "${temp_dir}$timestring.pnm";
    #my $hist  = "${temp_dir}${timestring}_hist.pnm";
    #my $histo = "${pickup_dir}${timestring}_histo.png";

#    `jpegtopnm $img > $pnm 2>/dev/null`;
    #`pnmhistmap $pnm |convert -scale 256!x100! - $histo`;

    #unlink ("$pnm");
    #unlink ("$hist");
    #&makeApacheOwned( "$histo" );
} # makehistogram

1;

#EOF
