package RSP;

use strict;
use warnings;
use File::stat;

our $VERSION = '0.1';
my $DEBUG = 0;

use base 'Exporter';
our @EXPORT = qw(rsp_validate rsp_plotstring merge_plotstrings write_plotfile rsp_average);

sub rsp_validate {
	return "Need RSP file." unless (my $rsp_file = shift);

	# Check if file exists
	return "RSP file $rsp_file does not exist." unless (-e $rsp_file);

	# Get size of RSP file
	return "Can't get file stats on $rsp_file" unless
			my $file_stat = stat($rsp_file);
	my $file_size = $file_stat->size;

	# RSP file shouldn't be more than 500 Bytes
	# (It's only 15+ numbers, after all...)
	return "RSP file $rsp_file too large." unless ($file_size < 500);

	my $line_cnt = 0;
	open RSPFH, "$rsp_file" or die $!;
	while (<RSPFH>) {
		# Check if the lines are of the required format, e.g.
		# 3 1.24059 -0.535859 0.296018 -0.000752861

		# See if first field is an integer
		return "Expecting leading digit in RSP file $rsp_file." unless \
				m/^(\d)\ /;

		# Check if we have between 2 and 7 floats after the integer
		#FIXME: This is not working.
		return "Need float coefficients in RSP file $rsp_file." unless \
				(/^\d( -?\d+\.\d+e?-?\d{2}){2,7}$/);
		$line_cnt++;
	}
	close( RSPFH );

	# We need exactly three line, no more and no less.
	return "Got $line_cnt lines in RSP file $rsp_file" unless ($line_cnt == 3);

	return;
}

sub rsp_plotstring() {
	my $elements = shift;

	$elements =~ s/^(\d)\s//;
	my $order = $1;
	my @coeffs = split(' ', $elements);
	my @rev_coeffs = reverse(@coeffs);

	my @powers = ();
	for (my $i=$#rev_coeffs ; $i>=0 ; $i--) {
		push( @powers, "$rev_coeffs[$i]*x**$i" );
	}
	my $ps = join('+', @powers);
	$ps =~ s/\+\-/\-/g;

	return $ps;
}

sub merge_plotstrings() {
	my @plot_strings = ();
	foreach (@_) {
		push( @plot_strings, $_ );
	}
	my $plot_cmd = 'plot ';
	my $joined_strings = join(', ', @plot_strings);
	$plot_cmd .= $joined_strings;

	return $plot_cmd;
}

sub rsp_unique() {
    my @sorted = sort @_;

	my $prev = 'nonesuch';
    my @unique = grep($_ ne $prev && (($prev) = $_), @sorted);

    return @unique;
}

sub rsp_average() {
    #TODO: verify this
	my $reflines = shift;
	my @lines = @{$reflines};
	my @total;
	my $cnt = 0;
	my $i;
	foreach my $line (@lines) {
		$line =~ s/^(\d)\s//;
		my $order = $1;
		#print "order: $order\n";
		my @coefficients = split(' ', $line);
		my @reversed = reverse(@coefficients);
		for ($i=0 ; $i<=$#reversed ; $i++) {
			$total[$i] += $reversed[$i];
		}
		$cnt++;
	}
	my @average;
	for ($i=0 ; $i<=$#total ; $i++) {
		my $field = sprintf "%.6f", $total[$i] / $cnt;
		$field =~ s/0+$//;
		$average[$i] = $field;
	}
	my $reversedaverage = $#average . " " . join(' ', reverse(@average));
	$DEBUG && print "average: $reversedaverage\n";
	return $reversedaverage;
}

1;

#EOF
